from random import randrange
from logging import getLogger
from . import models

logger = getLogger(__name__)


def get_current_app_from_request(request):
    try:
        api_key = request.META["HTTP_API_KEY"]
        app = models.App.objects.get(api_key=api_key, enabled=True)
        return app or None

    except Exception as e:
        logger.error(e)
        return None



"""companies URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/1.10/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  url(r'^$', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  url(r'^$', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.conf.urls import url, include
    2. Add a URL to urlpatterns:  url(r'^blog/', include('blog.urls'))
"""
from django.conf.urls import url, include
from django.contrib import admin
from rest_framework import routers
from .views import (
    AppViewSet,
    OrganizationalUnitTypeViewSet,
    OrganizationalUnitViewSet
)

router = routers.DefaultRouter()

router.register(r'apps', AppViewSet)
router.register(r'organizationalunittypes', OrganizationalUnitTypeViewSet)
router.register(r'organizationalunits', OrganizationalUnitViewSet)

urlpatterns = [
    url(r'^admin/', admin.site.urls),
    url(r'^api/', include(router.urls)),
    url(r'^api-auth/',
        include('rest_framework.urls', namespace='rest_framework'))
]



"""API Control serializers."""
from rest_framework import serializers
from .models import (
    App,
    OrganizationalUnitType,
    OrganizationalUnit
)


class OrganizationalUnitSerializer(serializers.ModelSerializer):
    class Meta:
        model = OrganizationalUnit
        fields = ('name', 'description', 'unit_type',
                  'created_at', 'updated_at')


class OrganizationalUnitTypeSerializer(serializers.ModelSerializer):

    class Meta:
        model = OrganizationalUnitType
        fields = ('name', 'description', 'app', 'parent', 'units')
        read_only_fields = ('units',)


class AppSerializer(serializers.ModelSerializer):

    class Meta:
        model = App



"""API Control models."""
import uuid
from django.conf import settings
from django.db import models
from django.db.models.signals import pre_save
from .utils import generate_code


class Responsible(models.Model):
    """Responsible: person who manage an app or group."""

    name = models.CharField(max_length=30, blank=True, null=True)
    email = models.EmailField(unique=True)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    def __str__(self):
        return self.email


class App(models.Model):
    """App model: One Instance per Client(business or App) that connects."""

    name = models.CharField(max_length=30)
    logo = models.URLField(blank=True, null=True)
    api_key = models.UUIDField(unique=True, default=uuid.uuid4, editable=False)
    enabled = models.BooleanField(default=False)
    owner = models.ForeignKey(settings.AUTH_USER_MODEL)

    metadata = models.JSONField()

    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    def __str__(self):
        return self.name


class OrganizationalUnitType(models.Model):
    """Groups type for an App(example: department)."""

    name = models.CharField(max_length=30)
    description = models.CharField(max_length=30, blank=True)
    app = models.ForeignKey(App)
    parent = models.ForeignKey('self', blank=True, null=True,
                               related_name='children')

    def __str__(self):
        return self.name


class OrganizationalUnit(models.Model):
    """Group for an App(example: accounting department)."""

    name = models.CharField(max_length=30)
    description = models.CharField(max_length=30, blank=True)
    unit_type = models.ForeignKey(OrganizationalUnitType, related_name="units")
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    responsibles = models.ManyToManyField(Responsible)

    def __str__(self):
        return self.name



"""API Control views."""
from rest_framework import viewsets
from .models import (
    App,
    OrganizationalUnitType,
    OrganizationalUnit
)
from .serializers import (
    AppSerializer,
    OrganizationalUnitTypeSerializer,
    OrganizationalUnitSerializer
)


class AppViewSet(viewsets.ModelViewSet):
    queryset = App.objects.all()
    serializer_class = AppSerializer


class OrganizationalUnitTypeViewSet(viewsets.ModelViewSet):
    queryset = OrganizationalUnitType.objects.all()
    serializer_class = OrganizationalUnitTypeSerializer


class OrganizationalUnitViewSet(viewsets.ModelViewSet):
    queryset = OrganizationalUnit.objects.all()
    serializer_class = OrganizationalUnitSerializer



"""API Control Django admin."""
from django.contrib import admin
from .models import (
    Responsible,
    App,
    OrganizationalUnitType,
    OrganizationalUnit
)


class AppAdmin(admin.ModelAdmin):
    """Django admin definition for App Model."""

    readonly_fields = ('api_key',)


admin.site.register(Responsible)
admin.site.register(App, AppAdmin)
admin.site.register(OrganizationalUnitType)
admin.site.register(OrganizationalUnit)



# -*- coding: utf-8 -*-

__title__ = 'djangorestframework-apicontrol'
__version__ = '0.3.1'
__author__ = 'Marcelo Cueto'
__license__ = 'GPLv3'
__copyright__ = 'Copyright 2017-2018 Marcelo Cueto'

# Version synonym
VERSION = __version__



# from distutils.core import setup
from setuptools import setup

setup(
    name='rest_framework_apicontrol',
    packages=['rest_framework_apicontrol'],
    version='0.3.1',
    description='Django Rest Framework Library to control Client Apps over APIs',
    author='Marcelo Cueto',
    author_email='cueto@live.cl',
    url='https://github.com/mcueto/djangorestframework-apicontrol',
    download_url='https://github.com/mcueto/djangorestframework-auth0/tarball/0.3.1',
    keywords=['api', 'control', 'rest framework', 'django'],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet :: WWW/HTTP',
    ],
    install_requires=[
        'django>=1.10.0',
        'djangorestframework>=1.9.0'
    ],
)



"""API Control permissions."""
from __future__ import unicode_literals
from logging import getLogger
from rest_framework.permissions import BasePermission
from .models import App

logger = getLogger(__name__)


class HasApiKeyPermission(BasePermission):
    """Allow request only when the header has a valid api_key."""

    def has_permission(self, request, view):
        """Validate if an App with the Api-Key header exists and is enabled."""
        if request.method == 'OPTIONS':
            return True

        try:
            api_key = request.META["HTTP_API_KEY"]
            app = App.objects.get(api_key=api_key, enabled=True)
            if(app):
                return True

        except Exception as e:
            logger.error(e)
            return False
